/*
 *  Copyright (c) 2006           Ji YongGang <jungle@soforge.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/***************************************************************************
 *   Copyright (C) 2004 by zhong                                           *
 *   zhongz@163.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
 
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "chmsee.h"
#include "bookmarks.h"

static void chmsee_delete_bookmarks_item_go(const gchar *, GSList *);
static GSList *chmsee_delete_bookmarks_item(const gchar *, gchar *);
static void chmsee_save_bookmarks(ChmSee *);
static void dialog_ok_button(GtkWidget *, ChmSee *);
static void dialog_bookmark_entry_changed(GtkEntry *, ChmSee *);

static void 
chmsee_delete_bookmarks_item_go(const gchar *path, GSList *list)
{
        FILE    *fp;
        GSList  *node = NULL;

        fp = fopen(path, "w");
        
        if (!fp) {
                return;
        }

        for (node = list; node; node = node->next) {
                fprintf(fp, (gchar *) node->data);
                g_free(node->data);
        }
        fclose(fp);

        g_slist_free(node);
        g_slist_free(list);
}

static GSList * 
chmsee_delete_bookmarks_item(const gchar *path, gchar *item_id)
{
        gchar   *ch;
        gchar   *get_tmp = NULL;
        gchar   *get_id = NULL;
        gchar   *check_id = NULL;
        gchar   *get_chm = NULL;
        gchar   *get_title = NULL;
        gchar   *get_path = NULL;
        FILE    *fp;
        char    lineBuf[256];
        int             label = 0;
        int             lastitem = 1;
        GSList  *list = NULL;

        fp = fopen(path, "r");
        
        if (!fp) {
                return (list);
        }
        
        while(fgets(lineBuf,512,fp)!=NULL) {
                
                if (( ch = g_strrstr (lineBuf,"["))) {
                        get_id = g_strdup(ch);
                        get_tmp = g_strdup( ch + 1);
                        check_id = g_strndup(get_tmp, strlen(get_tmp)-2);
                        label = 1;
                }
                
                if (!(g_strcasecmp(check_id, item_id) == 0)) {
                        
                        if ((ch = g_strrstr(lineBuf, "CHM="))) {

                                get_chm = g_strdup(ch);
                                label++;
                        } else if ((ch = g_strrstr(lineBuf, "TITLE="))) {

                                get_title = g_strdup(ch);
                                label++;
                        } else if ((ch = g_strrstr(lineBuf, "PATH="))) {

                                get_path = g_strdup(ch);
                                label++;
                        }
                }

                if (label == 4 ) {

                        list = g_slist_append(list, get_id);
                        list = g_slist_append(list, get_chm);
                        list = g_slist_append(list, get_title);
                        list = g_slist_append(list, get_path);
                        get_tmp = NULL;
                        get_id = NULL;
                        check_id = NULL;
                        get_chm = NULL;
                        get_title = NULL;
                        get_path = NULL;
                        label = 0;
                        lastitem++;
                }
        }

        fclose(fp);
        g_free(get_tmp);
        g_free(get_id);
        g_free(get_chm);
        g_free(get_title);
        g_free(get_path);
        g_free(check_id);

        return (list);
}

static void 
chmsee_save_bookmarks(ChmSee *chmsee)
{
        FILE    *file;
        gchar   *title;
        gchar   *path;
        gchar   *md5;

        md5 = g_strdup_printf("[%s]\n", chmsee->book_md5);        
        title = g_strdup_printf("TITLE=%s\n", chmsee->bookmark_title);
        path = g_strdup_printf("PATH=%s\n", chmsee->bookmark_uri);

        file = fopen(chmsee->bookmarks, "a+");
        fprintf(file, md5);
        fprintf(file, title);
        fprintf(file, path);

        g_free(md5);
        g_free(title);
        g_free(path);

        fclose(file);
}

static void 
dialog_ok_button(GtkWidget *widget, ChmSee *chmsee)
{
        chmsee_bookmark_treeview_add(chmsee->nb_bookmark_treeview, chmsee);
        gtk_widget_destroy(gtk_widget_get_toplevel (widget));
}

static void 
dialog_bookmark_entry_changed(GtkEntry *entry, ChmSee *chmsee)
{
        GtkWidget *ok_button;
        const gchar *text;
        gint text_length;
        
        text = gtk_entry_get_text(GTK_ENTRY (entry));
        text_length = strlen(text);
        chmsee->bookmark_title = (gchar *) text;

        ok_button = g_object_get_data(G_OBJECT (gtk_widget_get_toplevel((GtkWidget *)entry)), "ok_button");
        if (text_length >= 2) {
                gtk_widget_set_sensitive(ok_button, TRUE);

        } else {
                gtk_widget_set_sensitive(ok_button, FALSE);
        }
}

void 
chmsee_bookmark_treeview_add(GtkWidget *widget, ChmSee *chmsee)
{
        GtkTreeModel *model;
        GtkTreeIter iter;

        model = gtk_tree_view_get_model(GTK_TREE_VIEW (chmsee->nb_bookmark_treeview));

        LOG_DEBUG("bookmark_title = %s", chmsee->bookmark_title);
        LOG_DEBUG("bookmark_uri = %s", chmsee->bookmark_uri);

        if (chmsee->bookmark_uri) {
                gtk_list_store_append(GTK_LIST_STORE (model), &iter);
                gtk_list_store_set (GTK_LIST_STORE (model),
                                    &iter,
                                    TITLE_COLUMN, chmsee->bookmark_title,
                                    FILENAME_COLUMN, chmsee->bookmark_uri,
                                    ID_COLUMN, chmsee->book_md5,
                                    -1);
                 
                chmsee_save_bookmarks(chmsee);
        }
}

void
chmsee_bookmark_treeview_remove(GtkWidget *widget, ChmSee *chmsee)
{
        GtkTreeIter iter;
        GtkTreeModel *model;
        GtkTreeSelection *selection;
        GValue id_value = {0,};
        const gchar *path;
        gchar *item_id;

        selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (chmsee->nb_bookmark_treeview));

        if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
                path = (const gchar *) chmsee->bookmarks;

                gtk_tree_model_get_value (model, &iter, ID_COLUMN, &id_value);
                item_id = g_strdup_printf("%s", g_value_get_string (&id_value));
                gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
                chmsee_delete_bookmarks_item_go(path, chmsee_delete_bookmarks_item(path, item_id));

                g_value_unset (&id_value);
                g_free(item_id);
        }
}

void 
chmsee_load_bookmarks(ChmSee *chmsee)
{
        gchar   *get_tmp;
        gchar   *get_filename = NULL;
        gchar   *get_title = NULL;
        gchar   *get_path = NULL;
        gchar   *get_md5 = NULL;
        gchar   *ch;
        FILE    *fp;
        char    lineBuf[256];
        int     label = 0;
        
        fp = fopen(chmsee->bookmarks, "r");
        
        if (!fp) {
                return;
        }

        while (fgets(lineBuf, 512, fp) != NULL) {
                
                if (( ch = g_strrstr (lineBuf, "["))) {
                        get_tmp = g_strdup( ch + 1);
                        get_md5 = g_strndup(get_tmp, strlen(get_tmp)-2);
                        label = 1;
                        get_filename = NULL;
                        get_title = NULL;
                        get_path = NULL;
                }
                
                if ((ch = g_strrstr(lineBuf, "CHM="))) {
                        
                        get_tmp = g_strdup(ch + 4);
                        get_filename = g_strndup(get_tmp, strlen(get_tmp)-1);
                        label = 2;
                                                
                } else if ((ch = g_strrstr(lineBuf, "TITLE="))) {
                                
                        get_tmp = g_strdup(ch + 6);
                        get_title = g_strndup(get_tmp, strlen(get_tmp)-1);
                        label = 3;
                } else if ((ch = g_strrstr(lineBuf, "PATH="))) {
                                
                        get_tmp = g_strdup(ch + 5);
                        get_path = g_strndup(get_tmp, strlen(get_tmp)-1);
                        label = 4;
                }
                
                if (label ==4 ) {
                        
                        if (g_strcasecmp(chmsee->book_md5, get_md5) == 0) {
                                
                                GtkTreeModel *model;
                                GtkTreeIter iter;
                                
                                model = gtk_tree_view_get_model(GTK_TREE_VIEW (chmsee->nb_bookmark_treeview));

                                gtk_list_store_append(GTK_LIST_STORE (model), &iter);
                                gtk_list_store_set (GTK_LIST_STORE (model),
                                                    &iter,
                                                    TITLE_COLUMN, get_title,
                                                    FILENAME_COLUMN, get_path,
                                                    ID_COLUMN, get_md5,
                                                    -1);
                        }
                        
                        label = 0;
                }
                        
        }
        
        fclose(fp);
        g_free(get_filename);
        g_free(get_md5);
        g_free(get_title);
        g_free(get_path);
}

void 
bookmark_dialog_new(ChmSee *chmsee)
{
        GladeXML *glade;
        GtkWidget *bookmark_dialog;
        GtkWidget *bookmark_dialog_entry;
        GtkWidget *cancel_button;
        GtkWidget *ok_button;

        /* create charset dialog */
        glade = glade_xml_new(CHMSEE_DATA_DIR"/"GLADE_FILE, "bookmark_dialog", NULL);

        bookmark_dialog = glade_xml_get_widget(glade, "bookmark_dialog");
        g_signal_connect_swapped((gpointer) bookmark_dialog,
                                 "destroy",
                                 G_CALLBACK (gtk_widget_destroy),
                                 GTK_OBJECT (bookmark_dialog));

        /* bookmark name entry */
        bookmark_dialog_entry = (GtkWidget *)glade_xml_get_widget(glade, "bookmark_dialog_entry");
        gtk_entry_set_text(GTK_ENTRY (bookmark_dialog_entry), chmsee->bookmark_title);

        g_signal_connect(bookmark_dialog_entry, 
                         "changed", 
                         G_CALLBACK (dialog_bookmark_entry_changed),
                         chmsee);
                                        
        cancel_button = (GtkWidget *)glade_xml_get_widget(glade, "bookmark_dialog_cancelbutton");

        g_signal_connect_swapped((gpointer) cancel_button,
                                 "clicked",
                                 G_CALLBACK (gtk_widget_destroy),
                                 GTK_OBJECT (bookmark_dialog));

        ok_button = (GtkWidget *)glade_xml_get_widget(glade, "bookmark_dialog_okbutton");
        g_object_set_data(G_OBJECT (bookmark_dialog), "ok_button", ok_button);
        g_signal_connect((gpointer) ok_button,
                         "clicked",
                         G_CALLBACK (dialog_ok_button),
                         chmsee);
}
