/*
 *  Copyright (c) 2006           Ji YongGang <jungle@soforge.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/***************************************************************************
 *   Copyright (C) 2004 by zhong                                           *
 *   zhongz@163.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
 
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "chmsee.h"
#include "chmsee_config.h"
#include "charset.h"
#include "chmfile.h"
#include "bookmarks.h"

static void on_charset_selection(GtkTreeSelection *, ChmSee *);
static void on_chmsee_char_okbtn(GtkWidget *, ChmSee *);
static void char_setup_treeview(ChmSee *);

typedef struct
{
        const gchar    *cchar_number;
        const gchar    *cchar_codeset;
} ChmseeChar;

static ChmseeChar allchar[] =
{
        { N_("Simplified Chinese (GB18030)"), "GB18030" },
        { N_("Simplified Chinese (GB2312)"), "GB2312" },
        { N_("Simplified Chinese (GBK)"), "GBK" },
        { N_("Traditional Chinese (Big5)"), "BIG5" },
        { N_("Western (ISO-8859-1)"), "ISO-8859-1" },
        { N_("Western (ISO-8859-15)"), "ISO-8859-15" },
        { N_("Western (Windows-1252)"), "Windows-1252" },
        { N_("Japanese (Shift-JIS)"), "shift-jis" }
};

static void
on_charset_selection(GtkTreeSelection *selection, ChmSee *chmsee)
{
        GtkTreeIter iter;
        GtkTreeModel *model;
        GValue id_value = {0,};
        
        if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
                gtk_tree_model_get_value(model, &iter, FILENAME_COLUMN, &id_value);
                chmsee->current_codeset = g_strdup_printf("%s", g_value_get_string (&id_value));

                g_value_unset (&id_value);
        }
}

static void 
on_chmsee_char_okbtn(GtkWidget *widget, ChmSee *chmsee)
{
        if (chmsee_go_ex_chm_file(chmsee, chmsee->chm_name)) {
                chmsee_load_bookmarks(chmsee);
        }

        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (chmsee->char_checkbtn))) {
                chmsee->is_always = 1;
        } else {
                chmsee->is_always = 0;
        }
                
        gtk_widget_destroy(gtk_widget_get_toplevel (widget));
}

static void
char_setup_treeview(ChmSee *chmsee)
{
        GtkListStore *char_store;
        GtkTreeSelection *char_selection;
        GtkCellRenderer *char_cell;
        GtkTreeViewColumn *char_column;
        gint i;
        GtkTreeIter iter;

        char_store = gtk_list_store_new (NUM_COLUMNS, 
                                         G_TYPE_STRING, 
                                         G_TYPE_STRING, 
                                         G_TYPE_STRING);

        gtk_tree_view_set_model(GTK_TREE_VIEW (chmsee->char_treeview), 
                                GTK_TREE_MODEL(char_store));

        char_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (chmsee->char_treeview));
        gtk_tree_selection_set_mode (GTK_TREE_SELECTION (char_selection),
                                     GTK_SELECTION_SINGLE);
        g_signal_connect(G_OBJECT (char_selection),
                         "changed",
                         G_CALLBACK (on_charset_selection),
                         chmsee);

        char_cell = gtk_cell_renderer_text_new ();
        char_column = gtk_tree_view_column_new_with_attributes ("Charset",
                                                                char_cell,
                                                                "text", TITLE_COLUMN,
                                                                NULL);

        gtk_tree_view_append_column (GTK_TREE_VIEW (chmsee->char_treeview),
                                     GTK_TREE_VIEW_COLUMN (char_column));

        for (i=0;i < G_N_ELEMENTS (allchar); i++) {
        
                gtk_list_store_append(GTK_LIST_STORE(char_store), &iter);
                gtk_list_store_set(GTK_LIST_STORE(char_store),
                                   &iter,
                                   TITLE_COLUMN, allchar[i].cchar_number,
                                   FILENAME_COLUMN, allchar[i].cchar_codeset,
                                   -1);
        }
}

void 
chmsee_char_dialog_new(ChmSee *chmsee)
{
        GladeXML *glade;
        GtkWidget *dialog;
        GtkWidget *chmsee_char_okbtn;

        /* create charset dialog */
        glade = glade_xml_new(CHMSEE_DATA_DIR"/"GLADE_FILE, "charset_dialog", NULL);
        
        dialog = glade_xml_get_widget(glade, "charset_dialog");
        g_signal_connect_swapped((gpointer) dialog,
                                 "destroy",
                                 G_CALLBACK (gtk_widget_destroy),
                                 GTK_OBJECT (dialog));

        chmsee->char_treeview = (GtkWidget *)glade_xml_get_widget(glade, "char_treeview");

        chmsee->char_checkbtn = glade_xml_get_widget(glade, "char_checkbtn");

        if (chmsee->is_always == 1) {
                gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (chmsee->char_checkbtn), TRUE);
        }

        chmsee_char_okbtn = glade_xml_get_widget(glade, "char_okbtn");
        g_signal_connect ((gpointer) chmsee_char_okbtn, 
                          "clicked",
                          G_CALLBACK (on_chmsee_char_okbtn),
                          chmsee);

        char_setup_treeview(chmsee);
}
