/*
 *  Copyright (c) 2006           Ji YongGang <jungle@soforge.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gtkhtml/gtkhtml.h>

#if defined(__linux__)
#include <stdio.h>
#include <fcntl.h>
#include <openssl/md5.h>
#else
#include <md5.h>
#endif

#include "chmsee.h"
#include "utils.h"
#include "chmfile.h"
#include "chmsee_config.h"
#include "exchm.h"
#include "parse_hh.h"
#include "topuri.h"
#include "bookmarks.h"
#include "html.h"
#include "charset.h"

static void create_errord(void);

#if defined(__linux__)
static char *
MD5File(const char *filename, char *buf)
{
        unsigned char buffer[1024];
        unsigned char digest[16];
        static const char hex[]="0123456789abcdef";

        MD5_CTX ctx;
        int f, i;

        if(filename == NULL)
                return NULL;

        MD5_Init(&ctx);
        f = open(filename, O_RDONLY, 0666);
        if (f < 0)
                return NULL;

        while ((i = read(f, buffer, sizeof(buffer))) > 0)
                MD5_Update(&ctx, buffer, (unsigned int)i);

        close(f);

        if (i < 0)
                return NULL;
        
        if (buf == NULL)
                buf = malloc(33);
        if (buf == NULL)
                return (NULL);

        MD5_Final(digest, &ctx);

        for (i = 0; i < 16; i++) {
                buf[i+i] = hex[(u_int32_t)digest[i] >> 4];
                buf[i+i+1] = hex[digest[i] & 0x0f];
        }

        buf[i+i] = '\0';
        return (buf);
}
#endif

static void 
create_errord(void)
{
        GtkWidget *dialog;
        dialog = gtk_message_dialog_new(NULL,
                                        GTK_DIALOG_DESTROY_WITH_PARENT,
                                        GTK_MESSAGE_ERROR,
                                        GTK_BUTTONS_CLOSE,
                                        _("Open file failed"));

        g_signal_connect_swapped(GTK_OBJECT (dialog), 
                                 "response",
                                 G_CALLBACK (gtk_widget_destroy),
                                 GTK_OBJECT (dialog));

        gtk_widget_show(dialog);
}

void
command_delete_tmpdir(char *s_path)
{
        if (!g_file_test (s_path, G_FILE_TEST_EXISTS)) {
                return;
        }

        if (s_path != NULL) {
                char *argv[4];
                
                argv[0] = "rm";
                argv[1] = "-rf";
                argv[2] = s_path;
                argv[3] = NULL;

                g_spawn_async(g_get_tmp_dir (), argv, NULL, 
                              G_SPAWN_SEARCH_PATH, 
                              NULL, NULL, NULL,
                              NULL);
        }
}

gboolean 
chmsee_go_ex_chm_file(ChmSee *chmsee, const gchar *filename)
{
        gchar *file_ext;
        gchar *filename_utf8;
        gchar *title;
        gchar *chmdir;

        file_ext = g_strdup_printf("%s", g_strrstr(g_path_get_basename(filename), "."));
        
        if (!file_ext) {
                file_ext = g_strdup_printf("%s", g_strrstr(filename, "."));
        }

        if (g_ascii_strcasecmp(".chm", g_ascii_strdown(file_ext, -1)) == 0) {
                /* generate MD5 as extract chm folder name */
                chmsee->book_md5 = MD5File(filename, NULL);
                LOG_DEBUG ("book md5 value = %s", chmsee->book_md5);

                chmsee->book_dir = g_strdup_printf("%s/%s", chmsee->cache_dir, chmsee->book_md5);

                /* test extracted chm file */
                if (!g_file_test(chmsee->book_dir, G_FILE_TEST_IS_DIR)) {
                        ex_chm_file(chmsee, filename);
                        save_hhc_name(chmsee);
                } else {
                        chmsee->hhc_name = load_hhc_name(chmsee);
                }
                
                if (chmsee->hhc_name != NULL) {
                        parse_hhc_file(chmsee);
                } else {
                        chmsee->hhc_name = get_topurl(chmsee);

                        if (chmsee->hhc_name) {
                                load_file(chmsee, chmsee->hhc_name );
                                chmsee->home_url = g_strdup(chmsee->hhc_name);
                        } else {
                                create_errord();
                                return FALSE;
                        }
                }

                chmdir = g_strdup_printf("%s", g_path_get_dirname(filename));
                chmsee_save_last_chm_dir(chmsee, chmdir);

                /* update window title */
                filename_utf8 = convert_filename_to_utf8(filename, chmsee->current_codeset);
                title = g_strdup_printf("ChmSee - %s", g_path_get_basename(filename_utf8));
                gtk_window_set_title(GTK_WINDOW (chmsee->window), title);

                /* update toolbar status */
                gtk_widget_grab_focus(chmsee->chm_tree);
                gtk_widget_set_sensitive(chmsee->toolbar_small, TRUE);
                gtk_widget_set_sensitive(chmsee->toolbar_reset, TRUE);
                gtk_widget_set_sensitive(chmsee->toolbar_big, TRUE);
                gtk_widget_set_sensitive(chmsee->toolbar_search, TRUE);

                g_free(file_ext);
                g_free(chmdir);
                g_free(filename_utf8);
                g_free(title);

                return TRUE;
        }
        
        return FALSE;
}

gboolean 
open_chmfile(ChmSee *chmsee, const gchar *filename)
{
        GtkTreeModel *model;
        GtkTreeModel *bookmark_model;

        gboolean chmfile_opened = FALSE;

        /* clear previous treeview data */
        model = gtk_tree_view_get_model(GTK_TREE_VIEW (chmsee->chm_tree));
        gtk_tree_store_clear(GTK_TREE_STORE (model));

        bookmark_model = gtk_tree_view_get_model(GTK_TREE_VIEW (chmsee->nb_bookmark_treeview));
        gtk_list_store_clear(GTK_LIST_STORE (bookmark_model));

        chmsee->home_url = NULL;
        gtk_html_load_empty(GTK_HTML (chmsee->chm_html));
        chmsee->chm_name = g_strdup_printf("%s", filename);

        if (chmsee->is_always == 1) {
                chmfile_opened = chmsee_go_ex_chm_file(chmsee, filename);
        } else {
                chmsee_char_dialog_new(chmsee);
        }

        /* update toobar button status */
        if (chmfile_opened) {
                gtk_widget_set_sensitive(chmsee->toolbar_small, TRUE);
                gtk_widget_set_sensitive(chmsee->toolbar_reset, TRUE);
                gtk_widget_set_sensitive(chmsee->toolbar_big, TRUE);
                gtk_widget_set_sensitive(chmsee->toolbar_search, TRUE);

                if (chmsee->home_url) {
                        gtk_widget_show(chmsee->toolbar_home);
                        gtk_widget_set_sensitive(chmsee->toolbar_home, TRUE);
                }

                gtk_widget_grab_focus((GtkWidget*)chmsee->chm_tree);

                chmsee_load_bookmarks(chmsee);
        }

        return TRUE;
}

void 
load_file(ChmSee *chmsee, const gchar *str_uri) 
{
        gchar   *full_link;
        gchar   *ch;

        LOG_DEBUG ("load_file: open uri = %s", str_uri);

        // MS-ITS --> MS-ITS:chm::/link
        if ((ch = g_strrstr (str_uri, ":"))) {
                str_uri = g_strdup (ch + 1);
                LOG_DEBUG ("%s", str_uri);
        }

        if (str_uri[0] == '/') {
                full_link = g_strdup_printf("%s%s", chmsee->book_dir, str_uri);
        } else {
                full_link = g_strdup_printf("%s/%s", chmsee->book_dir, str_uri);
        }

        chmsee_html_open_uri(chmsee, full_link, 0);
        
        if (full_link != str_uri) {
                g_free (full_link);
        }
}
