/*
 *  Copyright (c) 2006           Ji YongGang <jungle@soforge.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/***************************************************************************
 *   Copyright (C) 2003 by zhong                                           *
 *   zhongz@163.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <gtkhtml/gtkhtml.h>

#include "chmsee.h"
#include "chmsee_config.h"
#include "html.h"
#include "setup.h"
#include "search.h"
#include "bookmarks.h"
#include "charset.h"
#include "chmfile.h"

static void chmsee_home_init(ChmSee *);
static void chmsee_init(ChmSee *);
static gboolean on_chmsee_delete(GtkWidget *, GdkEventAny *, ChmSee *);
static void on_chmsee_destroy(GtkWidget *, ChmSee *);
static void on_open_chmfile_ok(GtkWidget *, gint, ChmSee *);
static void on_open_chmfile_filesel(GtkWidget *, ChmSee *);
static void on_search(GtkWidget *, ChmSee *);
static void on_setup(GtkWidget *, ChmSee *);
static void on_hide_pw_button(GtkWidget *, ChmSee *);
static void on_show_pw_button(GtkWidget *, ChmSee *);
static void on_chm_back(GtkWidget *, ChmSee *);
static void on_chm_forward(GtkWidget *, ChmSee *);
static void go_home(GtkWidget *, ChmSee *);
static void on_chm_big(GtkWidget *, ChmSee *);
static void on_chm_reset(GtkWidget *, ChmSee *);
static void on_chm_small(GtkWidget *, ChmSee *);
static void on_bookmark_add(GtkWidget *, ChmSee *);
static void on_bookmark_remove(GtkWidget *, ChmSee *);
static void bookmark_entry_setext(ChmSee *);
static void bookmark_selection_cb(GtkTreeView *, GtkTreePath *, GtkTreeViewColumn *, ChmSee *);
static void on_contents_selection(GtkTreeSelection *, ChmSee *);
static void on_about(GtkWidget *);
static void nb_search_entry_changed(GtkEntry *, ChmSee *);
static void nb_bookmark_entry_changed (GtkEntry *, ChmSee *);
static void chmsee_setup_treeview(ChmSee *);


static void
chmsee_home_init(ChmSee *chmsee)
{
        chmsee->home = g_strdup_printf("%s/%s", g_get_home_dir(), ".chmsee");

        if (!g_file_test(chmsee->home, G_FILE_TEST_IS_DIR)) {
                mkdir(chmsee->home, 0777);
        }

        chmsee->cache_dir = g_strdup_printf("%s/%s", chmsee->home, "bookshelf");

        if (!g_file_test(chmsee->cache_dir, G_FILE_TEST_IS_DIR)) {
                mkdir(chmsee->cache_dir, 0777);
        }
}

static void 
chmsee_init(ChmSee *chmsee)
{
        gtk_window_set_title(GTK_WINDOW (chmsee->window), "ChmSee");
        gtk_widget_set_sensitive (chmsee->toolbar_back, FALSE);
        gtk_widget_set_sensitive (chmsee->toolbar_forward, FALSE);
        gtk_widget_set_sensitive (chmsee->toolbar_small, FALSE);
        gtk_widget_set_sensitive (chmsee->toolbar_reset, FALSE);
        gtk_widget_set_sensitive (chmsee->toolbar_big, FALSE);
        gtk_widget_set_sensitive (chmsee->toolbar_search, FALSE);
        gtk_widget_hide (chmsee->toolbar_home);

        chmsee->link = NULL;
        chmsee->link_pos = 0;
        chmsee->current_link = NULL;
        chmsee->home_url = NULL;
        chmsee->hhc_name = NULL;
        chmsee->chm_name = NULL;
        chmsee->bookmark_title = NULL;
        chmsee->bookmark_uri = NULL;

        chmsee->is_always = 0;
        chmsee->current_codeset = "UTF-8";

        chmsee_home_init(chmsee);
}


/* 
 * callbacks
 */

static gboolean
on_chmsee_delete(GtkWidget *widget, GdkEventAny *event, ChmSee *chmsee)
{
        gtk_widget_destroy(chmsee->window);

        return TRUE;
}

static void
on_chmsee_destroy(GtkWidget *widget, ChmSee * chmsee)
{
        chmsee_save_config(chmsee);
        gtk_main_quit();

        g_free(chmsee);
}

static void
on_open_chmfile_ok(GtkWidget *widget, gint response_id, ChmSee *chmsee)
{
        if (response_id == GTK_RESPONSE_OK)
        {
                gchar *filename;
                
                filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));

                if (filename)
                {
                        open_chmfile(chmsee, filename);
                        g_free(filename);
                }
        }

        gtk_widget_destroy(widget);
}

static void
on_open_chmfile_filesel(GtkWidget *widget, ChmSee *chmsee)
{
        GladeXML *glade;
        GtkWidget *dialog;
        GtkFileFilter *filter;

        gchar *dir_name;
        
        /* create openfile dialog */
        glade = glade_xml_new(CHMSEE_DATA_DIR"/"GLADE_FILE, "openfile_dialog", NULL);
        dialog = glade_xml_get_widget(glade, "openfile_dialog");

        g_signal_connect(dialog, 
                         "response",
                         GTK_SIGNAL_FUNC (on_open_chmfile_ok),
                         chmsee);

        /* file list fiter */
        filter = gtk_file_filter_new();
        gtk_file_filter_set_name(filter, _("CHM Files"));
        gtk_file_filter_add_pattern(filter, "*.chm");
        gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filter);

        filter = gtk_file_filter_new();
        gtk_file_filter_set_name(filter, _("All Files"));
        gtk_file_filter_add_pattern(filter, "*");
        gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filter);

        /* previous opened folder */
        dir_name = chmsee_load_last_chm_dir(chmsee);
        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER (dialog), dir_name);

        g_free(dir_name);
}

static void
on_search(GtkWidget *widget, ChmSee *chmsee)
{
        create_search_window(widget, chmsee);
}

static void
on_setup(GtkWidget *widget, ChmSee *chmsee)
{
        chmsee_setup_window_new(widget, chmsee);
}

static void
on_hide_pw_button(GtkWidget *widget, ChmSee *chmsee)
{
        gtk_widget_hide(chmsee->chm_pw);
        gtk_widget_show(chmsee->toolbar_showw);
}

static void
on_show_pw_button(GtkWidget *widget, ChmSee *chmsee)
{
        gtk_widget_show(chmsee->chm_pw);
        gtk_widget_hide(chmsee->toolbar_showw);
}

static void 
on_chm_back(GtkWidget *widget, ChmSee *chmsee)
{
        GList   *list = NULL;
        gchar   *url;

        chmsee->link_pos--;
        list = g_list_nth(chmsee->link, chmsee->link_pos-1);
        url = g_convert((const gchar *) list->data, -1, chmsee->current_codeset, "UTF-8", NULL, NULL, NULL);
        chmsee_html_open_uri(chmsee, url, 2);
}

static void 
on_chm_forward(GtkWidget *widget, ChmSee *chmsee)
{
        GList   *list = NULL;
        gchar   *url;
        chmsee->link_pos++;
        list = g_list_nth(chmsee->link, chmsee->link_pos-1);
        url = g_convert((const gchar *) list->data, -1, chmsee->current_codeset, "UTF-8", NULL, NULL, NULL);
        chmsee_html_open_uri(chmsee, url, 2);
}

static void
go_home(GtkWidget *widget, ChmSee *chmsee)
{
        load_file(chmsee, chmsee->home_url);
}

static void 
on_chm_big(GtkWidget *widget, ChmSee *chmsee)
{
        gtk_html_zoom_in(GTK_HTML (chmsee->chm_html));
}

static void 
on_chm_reset(GtkWidget *widget, ChmSee *chmsee)
{
        gtk_html_zoom_reset(GTK_HTML (chmsee->chm_html));
}

static void 
on_chm_small(GtkWidget *widget, ChmSee *chmsee)
{
        gtk_html_zoom_out(GTK_HTML (chmsee->chm_html));
}

static void
on_bookmark_add(GtkWidget *widget, ChmSee *chmsee)
{
        chmsee_bookmark_treeview_add(widget, chmsee);
}

static void
on_bookmark_remove(GtkWidget *widget, ChmSee *chmsee)
{
        chmsee_bookmark_treeview_remove(widget, chmsee);
}

static void 
bookmark_entry_setext(ChmSee *chmsee)
{
        gtk_entry_set_text(GTK_ENTRY(chmsee->nb_bookmark_entry), chmsee->bookmark_title);
}

static void
bookmark_selection_cb(GtkTreeView *tree_view,
                      GtkTreePath *path,
                      GtkTreeViewColumn *column,
                      ChmSee *chmsee)
{
        GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
        GtkTreeIter  iter;
        gchar *anchor;

        gtk_tree_model_get_iter(model, &iter, path);
        gtk_tree_model_get(model, &iter, FILENAME_COLUMN, &anchor, -1);
        load_file(chmsee, anchor);
}

static void
on_contents_selection(GtkTreeSelection *selection, ChmSee *chmsee)
{
        GtkTreeIter iter;
        GtkTreeModel *model;
        GValue title_value = {0,};
        GValue uri_value = {0,};
        gchar *title;
        gchar *uri;
        
        if (!gtk_tree_selection_get_selected(selection, &model, &iter)) {
                return;
        }

        LOG_INFO("on_contents_selection");
        
        gtk_tree_model_get_value(model, &iter, 
                                 TITLE_COLUMN, 
                                 &title_value);
        gtk_tree_model_get_value(model, &iter, 
                                 FILENAME_COLUMN, 
                                 &uri_value);

        title = g_strdup_printf("%s", g_value_get_string (&title_value));
        uri = g_strdup_printf("%s", g_value_get_string (&uri_value));

        /* set bookmark */
        chmsee->bookmark_title = g_strdup(title);
        chmsee->bookmark_uri = g_strdup(uri);
        bookmark_entry_setext(chmsee);

        /* open selected uri */
        if (g_ascii_strcasecmp(CHMSEE_NO_LINK, uri))
                load_file(chmsee, uri);

        g_value_unset(&title_value);
        g_value_unset(&uri_value);

        g_free(title);
        g_free(uri);
}

static void 
on_about(GtkWidget *widget)
{
        GladeXML *glade;
        GtkWidget *dialog;

        glade = glade_xml_new(CHMSEE_DATA_DIR"/"GLADE_FILE, "about_dialog", NULL);
        dialog = glade_xml_get_widget(glade, "about_dialog");

        gtk_window_set_modal(GTK_WINDOW (dialog), TRUE);

        gtk_about_dialog_set_version(GTK_ABOUT_DIALOG (dialog), PACKAGE_VERSION);
}

static void 
nb_search_entry_changed(GtkEntry *entry, ChmSee *chmsee) 
{
        const gchar *text;
        gint text_length;

        text = gtk_entry_get_text(GTK_ENTRY (entry));
        text_length = strlen(text);
        
        if (text_length >= 2) {
                gtk_widget_set_sensitive(chmsee->nb_button_contents, TRUE);
        } else {
                gtk_widget_set_sensitive(chmsee->nb_button_contents, FALSE);
        }
}

static void 
nb_bookmark_entry_changed (GtkEntry *entry, ChmSee *chmsee) 
{
        const gchar *text;
        gint text_length;

        text = gtk_entry_get_text(GTK_ENTRY (entry));
        text_length = strlen(text);
        chmsee->bookmark_title = (gchar *) text;

        if (text_length >= 2) {
                gtk_widget_set_sensitive (chmsee->nb_button_bookmark_add, TRUE);
        } else {
                gtk_widget_set_sensitive (chmsee->nb_button_bookmark_add, FALSE);
        }
}

static void
chmsee_setup_treeview(ChmSee *chmsee)
{
        GtkTreeStore *contents_store;
        GtkTreeSelection *selection;
        GtkCellRenderer *cell;
        GtkTreeViewColumn *column;

        GtkListStore *bookmark_store;
        GtkTreeSelection *bookmark_selection;
        GtkCellRenderer *bookmark_cell;
        GtkTreeViewColumn *bookmark_column;

        /* contents */
        contents_store = gtk_tree_store_new(NUM_COLUMNS, 
                                            G_TYPE_STRING, 
                                            G_TYPE_STRING, 
                                            G_TYPE_STRING);

        gtk_tree_view_set_model(GTK_TREE_VIEW (chmsee->chm_tree), 
                                GTK_TREE_MODEL (contents_store));

        selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (chmsee->chm_tree));
        gtk_tree_selection_set_mode(GTK_TREE_SELECTION(selection), GTK_SELECTION_SINGLE);

        g_signal_connect(G_OBJECT (selection),
                         "changed",
                         G_CALLBACK (on_contents_selection),
                         chmsee);

        cell = gtk_cell_renderer_text_new();
        column = gtk_tree_view_column_new_with_attributes("Tree",
                                                          cell,
                                                          "text", TITLE_COLUMN,
                                                          NULL);
        
        gtk_tree_view_append_column(GTK_TREE_VIEW (chmsee->chm_tree),
                                    GTK_TREE_VIEW_COLUMN (column));

        /* bookmarks */
        bookmark_store = gtk_list_store_new(NUM_COLUMNS, 
                                            G_TYPE_STRING, 
                                            G_TYPE_STRING, 
                                            G_TYPE_STRING);

        gtk_tree_view_set_model(GTK_TREE_VIEW (chmsee->nb_bookmark_treeview), 
                                GTK_TREE_MODEL(bookmark_store));

        bookmark_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (chmsee->nb_bookmark_treeview));
        gtk_tree_selection_set_mode (GTK_TREE_SELECTION (bookmark_selection),
                                     GTK_SELECTION_SINGLE);

        g_signal_connect(chmsee->nb_bookmark_treeview, 
                          "row_activated", 
                          G_CALLBACK (bookmark_selection_cb), 
                          chmsee);

        bookmark_cell = gtk_cell_renderer_text_new ();
        bookmark_column = gtk_tree_view_column_new_with_attributes ("Bookmarks",
                                                                    bookmark_cell,
                                                                    "text", TITLE_COLUMN,
                                                                    NULL);

        gtk_tree_view_append_column (GTK_TREE_VIEW (chmsee->nb_bookmark_treeview),
                                     GTK_TREE_VIEW_COLUMN (bookmark_column));

        g_signal_connect (chmsee->nb_bookmark_entry, 
                          "changed", 
                          G_CALLBACK (nb_bookmark_entry_changed),
                          chmsee);
}

ChmSee *
chmsee_new(void)
{
        ChmSee *chmsee;
        GladeXML *glade;

        GtkWidget *open_button;
        GtkWidget *setup_button;
        GtkWidget *about_button;
        GtkWidget *quit_button;
        GtkWidget *pixgui_button;
        GtkWidget *nb_button_bookmark_remove;
        GtkWidget *html_view_sw;

        chmsee = g_new0(ChmSee, 1);

        glade = glade_xml_new(CHMSEE_DATA_DIR"/"GLADE_FILE, "main_window", NULL);

        /* main window */
        chmsee->window = glade_xml_get_widget(glade, "main_window");
        g_object_set_data(G_OBJECT (chmsee->window), "ChmSee", chmsee);
        g_signal_connect(chmsee->window,
                         "delete_event",
                         G_CALLBACK (on_chmsee_delete),
                         chmsee);
        g_signal_connect(chmsee->window,
                         "destroy",
                         G_CALLBACK (on_chmsee_destroy),
                         chmsee);

        /* toolbar buttons */
        open_button = glade_xml_get_widget(glade, "toolbar_open");
        g_signal_connect(open_button,
                         "clicked",
                         G_CALLBACK (on_open_chmfile_filesel),
                         chmsee);

        setup_button = glade_xml_get_widget(glade, "toolbar_setup");
        g_signal_connect(setup_button,
                         "clicked",
                         G_CALLBACK (on_setup),
                         chmsee);

        about_button = glade_xml_get_widget(glade, "toolbar_about");
        g_signal_connect(about_button,
                         "clicked",
                         G_CALLBACK (on_about),
                         NULL);

        quit_button = glade_xml_get_widget(glade, "toolbar_quit");
        g_signal_connect(quit_button,
                         "clicked",
                         G_CALLBACK (on_chmsee_destroy),
                         chmsee);


        chmsee->toolbar_showw = glade_xml_get_widget(glade, "toolbar_showw");
        g_signal_connect(chmsee->toolbar_showw,
                         "clicked",
                         G_CALLBACK (on_show_pw_button),
                         chmsee);

        chmsee->toolbar_search = glade_xml_get_widget(glade, "toolbar_search");
        g_signal_connect(chmsee->toolbar_search,
                         "clicked",
                         G_CALLBACK (on_search),
                         chmsee);

        chmsee->toolbar_back = glade_xml_get_widget(glade, "toolbar_back");
        g_signal_connect(chmsee->toolbar_back,
                         "clicked",
                         G_CALLBACK (on_chm_back),
                         chmsee);

        chmsee->toolbar_forward = glade_xml_get_widget(glade, "toolbar_forward");
        g_signal_connect(chmsee->toolbar_forward,
                         "clicked",
                         G_CALLBACK (on_chm_forward),
                         chmsee);

        chmsee->toolbar_home = glade_xml_get_widget(glade, "toolbar_home");
        g_signal_connect(chmsee->toolbar_home,
                         "clicked",
                         G_CALLBACK (go_home),
                         chmsee);

        chmsee->toolbar_big = glade_xml_get_widget(glade, "toolbar_big");
        g_signal_connect(chmsee->toolbar_big,
                         "clicked",
                         G_CALLBACK (on_chm_big),
                         chmsee);

        chmsee->toolbar_reset = glade_xml_get_widget(glade, "toolbar_reset");
        g_signal_connect(chmsee->toolbar_reset,
                         "clicked",
                         G_CALLBACK (on_chm_reset),
                         chmsee);

        chmsee->toolbar_small = glade_xml_get_widget(glade, "toolbar_small");
        g_signal_connect(chmsee->toolbar_small,
                         "clicked",
                         G_CALLBACK (on_chm_small),
                         chmsee);

        
        chmsee->chm_pw = glade_xml_get_widget(glade, "chm_pw");

        pixgui_button = glade_xml_get_widget(glade, "pixgui_button");
        g_signal_connect(pixgui_button,
                         "clicked",
                         G_CALLBACK (on_hide_pw_button),
                         chmsee);

        /* notebook */
        chmsee->chm_tree = (GtkWidget *)glade_xml_get_widget(glade, "chm_tree");

        chmsee->nb_entry = glade_xml_get_widget(glade, "nb_entry");
  
        g_signal_connect (chmsee->nb_entry, "changed",
                          G_CALLBACK (nb_search_entry_changed),
                          chmsee);

        chmsee->nb_button_contents = glade_xml_get_widget(glade, "nb_button_contents");
        chmsee->nb_bookmark_entry = glade_xml_get_widget(glade, "nb_bookmark_entry");

        chmsee->nb_bookmark_treeview = (GtkWidget *)glade_xml_get_widget(glade, "nb_bookmark_treeview");

        chmsee->nb_button_bookmark_add = glade_xml_get_widget(glade, "nb_button_bookmark_add");
        g_signal_connect (chmsee->nb_button_bookmark_add,
                          "clicked",
                          G_CALLBACK (on_bookmark_add),
                          chmsee);

        nb_button_bookmark_remove = glade_xml_get_widget(glade, "nb_button_bookmark_remove");
        g_signal_connect (nb_button_bookmark_remove, 
                          "clicked",
                          G_CALLBACK (on_bookmark_remove),
                          chmsee);

        /* gtkhtml */
        chmsee->chm_html = chmsee_html_view_new(chmsee);
        gtk_widget_show (chmsee->chm_html);

        html_view_sw = glade_xml_get_widget(glade, "html_view_sw");
        gtk_container_add (GTK_CONTAINER (html_view_sw), chmsee->chm_html);

        /* initial notebook treeviews */
        chmsee_setup_treeview(chmsee);

        /* load config */
        chmsee_init(chmsee);
        chmsee_init_config_path(chmsee);
        chmsee_load_config(chmsee);     

        /* open chmsee info page */
        chmsee_html_open_uri(chmsee, CHMSEE_DATA_DIR"/"CHMSEE_HELP, 2);

        LOG_INFO ("Initializer Done.");

        return chmsee;
}
