/*
 *  Copyright (c) 2006           Ji YongGang <jungle@soforge.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/***************************************************************************
 *   Copyright (C) 2003 by zhong                                           *
 *   zhongz@163.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <stdio.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <string.h>
#include <gtk/gtk.h>
#include <libgnomevfs/gnome-vfs-utils.h>
#include <gdk/gdk.h>
#include <gtkhtml/gtkhtml.h>
#include <gtkhtml/gtkhtml-stream.h>

#include "chmsee.h"
#include "html.h"
#include "menu_popup.h"
#include "bookmarks.h"

static void open_uri_errord(void);
static void chmsee_html_url_requested(GtkHTML *, const char *, GtkHTMLStream *, ChmSee *);
static void chmsee_html_load_done(GtkWidget *);
static void chmsee_html_refresh(GtkWidget *);
static void chmsee_add_list (ChmSee *, gchar *, gint);
static gchar *chmsee_conv_uri(const gchar *);
static gchar *chmsee_html_split_uri(const gchar *, gchar **);
static void chmsee_html_link_clicked(GtkWidget *, const gchar *, ChmSee *);
static gboolean chmsee_html_button_pressed (GtkWidget *, GdkEventButton *, ChmSee *);

static void 
open_uri_errord()
{
        GtkWidget *dialog;
        dialog = gtk_message_dialog_new (NULL,
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_ERROR,
                                         GTK_BUTTONS_CLOSE,
                                         _("Wrong link"));

        g_signal_connect_swapped (GTK_OBJECT (dialog),
                                  "response",
                                  G_CALLBACK (gtk_widget_destroy),
                                  GTK_OBJECT (dialog));

        gtk_widget_show (dialog);
}

static void 
chmsee_html_url_requested(GtkHTML *html, const char *full_link, GtkHTMLStream *stream, ChmSee *chmsee)
{
        int     fp;
        gchar   *buffer;
        struct  stat stbuf;
        gchar   *conv_html_utf = NULL;
        gchar   *s;

        s = g_strrstr(g_strdown(g_path_get_basename(full_link)) , ".");
        
        if (s) {
                if ((g_strcasecmp(".htm", s) == 0) 
                    || (g_strcasecmp(".html", s) == 0) 
                    || (g_strcasecmp(".txt", s) == 0)) {

                        if (!lstat(full_link, &stbuf)) {
                                if (opendir(full_link)) {
                        
                                } else if (opendir(full_link) == NULL 
                                           && (fp = open(full_link, O_RDONLY | O_NONBLOCK))) {
                                        buffer = (gchar *) mmap(NULL, stbuf.st_size, PROT_READ, MAP_SHARED, fp, 0);
                                
                                        if (g_strcasecmp("NONE", chmsee->current_codeset) == 0 ) {
                                                conv_html_utf = g_strdup(buffer);
                                        } else {
                                                conv_html_utf = g_convert(buffer, -1, "UTF-8", chmsee->current_codeset, NULL, NULL, NULL);
                                        }

                                        if (conv_html_utf) {
                                                gtk_html_write(GTK_HTML (html), stream, conv_html_utf, -1);
                                        } else {
                                                gtk_html_write(GTK_HTML (html), stream, buffer, stbuf.st_size);
                                        }
                                
                                        close(fp);
                                        munmap(buffer, stbuf.st_size);
                                        gtk_html_end(GTK_HTML (html), stream, GTK_HTML_STREAM_OK);
                                        g_free(conv_html_utf);
                                }
                        }
                } else if ( (g_strcasecmp(".gif", s) == 0) 
                            || (g_strcasecmp(".gif;423615", s) == 0) /* ugly!!! only for apue2 */
                            || (g_strcasecmp(".jpg", s) == 0) 
                            || (g_strcasecmp(".jpeg", s) == 0) 
                            || (g_strcasecmp(".png", s) == 0) 
                            || (g_strcasecmp(".bmp", s) == 0 )) {
                                        
                        gchar *strdown = NULL;
                        strdown = g_convert(full_link, -1, chmsee->current_codeset, "UTF-8", NULL, NULL, NULL);

                        if (strdown) {
                                full_link = (const char *) strdown;
                        }
                
                        full_link = (const gchar *) g_strdown((gchar *) full_link);

                        if (!lstat(full_link, &stbuf)) {
                                if (opendir(full_link)) {
                        
                                } else if (opendir(full_link) == NULL 
                                            && (fp = open(full_link, O_RDONLY | O_NONBLOCK))) {
                           
                                        buffer = (gchar *) mmap(NULL, stbuf.st_size, PROT_READ, MAP_SHARED, fp, 0);
                                        gtk_html_write(GTK_HTML (html), stream, buffer, stbuf.st_size);
                                        close(fp);
                                        munmap(buffer, stbuf.st_size);
                                        gtk_html_end(GTK_HTML (html), stream, GTK_HTML_STREAM_OK);
                                }
                        }
                
                        g_free(strdown);
                }
        }
}

static void
chmsee_html_load_done(GtkWidget *html)
{
        gtk_widget_grab_focus(GTK_WIDGET (html));
}

static void 
chmsee_html_refresh(GtkWidget *html) 
{
        gtk_widget_draw(GTK_WIDGET (html), NULL);
}

// chmsee_add_list 函数功能未完善 .....
static void 
chmsee_add_list(ChmSee *chmsee, gchar *url, gint i)
{
        gchar *conv_url;
        
        conv_url = g_convert(url, -1, "UTF-8", chmsee->current_codeset, NULL, NULL, NULL);

        if ( i == 0 ) {
                chmsee->link = g_list_append (chmsee->link, conv_url);
                chmsee->link_pos = g_list_length (chmsee->link);
        } else if ( i == 1 ) {
                GList *list = NULL;
                list = g_list_nth(chmsee->link, g_list_length(chmsee->link) - 1);
                g_list_delete_link(chmsee->link, list);
                chmsee->link = g_list_append (chmsee->link, conv_url);
                chmsee->link_pos = g_list_length (chmsee->link);
        }
        
        g_free(conv_url);

        if (chmsee->link_pos > 1) {
                gtk_widget_set_sensitive (chmsee->toolbar_back, TRUE);
        } else {
                gtk_widget_set_sensitive (chmsee->toolbar_back, FALSE);
        }
        
        if (chmsee->link_pos == g_list_length (chmsee->link) )  {
                gtk_widget_set_sensitive (chmsee->toolbar_forward, FALSE);
        } else {
                gtk_widget_set_sensitive (chmsee->toolbar_forward, TRUE);
        }
}

static gchar *
chmsee_conv_uri(const gchar *uri)
{
        gchar *url;
        gchar *conv_url;

        LOG_DEBUG ("uri = %s", uri);
        url = g_strdup_printf("file://%s", uri);
        conv_url = gnome_vfs_get_local_path_from_uri(url);
        g_free(url);
        LOG_DEBUG ("conv_url = %s", conv_url);

        return (conv_url);
}

static gchar *
chmsee_html_split_uri(const gchar *uri, gchar **anchor)
{
        gchar *ret_val;
        gchar *ch;
        
        if ((ch = strchr(uri, '?')) || (ch = strchr(uri, '#'))) {
                if (anchor) {
                        *anchor = g_strdup(ch + 1);
                }
                
                ret_val = g_strndup(uri, ch - uri);
        } else {
                ret_val = g_strdup(uri);
        }

        return ret_val;
}

static void
chmsee_html_link_clicked(GtkWidget *html, const gchar *url, ChmSee *chmsee)
{       
        gchar   *full_link = NULL;
        gchar   *s;
        gchar   *anchor = NULL;
        gchar   *str;
        gint    i;

        full_link = chmsee_html_split_uri(url, &anchor);

        s = g_strrstr(g_strdown(g_path_get_basename(full_link)) , ".");
        
        GdkCursor       *cursor;
        cursor = gdk_cursor_new (GDK_WATCH);
        gdk_window_set_cursor (chmsee->chm_html->window, cursor);
        gdk_cursor_unref (cursor);
        
        if (s) {
                if ((g_strcasecmp(".htm", s) == 0) 
                    || (g_strcasecmp(".html", s) == 0) 
                    || (g_strcasecmp(".txt", s) == 0)) {

                        str = g_strndup(full_link, strlen(chmsee->book_dir));

                        if (g_strcasecmp(chmsee->book_dir, str) == 0) {
                        
                                if (chmsee->link_pos == (g_list_length(chmsee->link) - 1)) {
                                        i = 1;
                                } else {
                                        i = 0;
                                }

                                full_link = g_convert(url, -1, chmsee->current_codeset, "UTF-8", NULL, NULL, NULL);

                                if (full_link) {
                                        chmsee_html_open_uri (chmsee, (const gchar *)full_link, i);
                                } else {
                                        chmsee_html_open_uri (chmsee, url, i);
                                }
                        }
                }
        } else {
                
                if (anchor) {
                        gtk_html_jump_to_anchor (GTK_HTML(html), anchor);
                }
        }
        
        gdk_window_set_cursor (chmsee->chm_html->window, NULL);
}

static gboolean 
chmsee_html_button_pressed(GtkWidget *widget, GdkEventButton *event, ChmSee *chmsee)
{
        if (event->type == GDK_BUTTON_PRESS) {
                if (event->button == 3) {
                        chmsee_menu_popup (event, chmsee);
                }
        }
        return FALSE;
}

void
chmsee_html_bookmark_add(GtkWidget *widget, ChmSee *chmsee)
{
        const gchar *html_title;

        if (chmsee->bookmark_uri) {
                 
                chmsee->bookmark_uri =  g_strdup(chmsee->current_link + strlen(chmsee->book_dir));
                // strstr(g_strrstr(chmsee->current_link, g_path_get_basename(chmsee->book_dir)) , "/");
                if (g_strrstr(chmsee->current_link, g_path_get_basename(chmsee->book_dir))) {

                        html_title = gtk_html_get_title(GTK_HTML(chmsee->chm_html));
                
                        if (strlen(html_title) == 0) {
                        
                                html_title = g_strdup_printf("%s", "New Bookmark item");
                        }
                
                        gtk_entry_set_text(GTK_ENTRY(chmsee->nb_bookmark_entry), html_title);
                        chmsee->bookmark_title = g_strdup(html_title);
                
                        chmsee->bookmark_uri =  g_strdup(chmsee->current_link + strlen(chmsee->book_dir));
                } else {

                        return;
                        // chmsee->bookmark_uri =  g_strdup_printf("%s", "nolink");
                }

                bookmark_dialog_new(chmsee);
        }
}

void
chmsee_html_open_uri(ChmSee *chmsee, const gchar *str_uri, gint i)
{
        gchar *full_link;
        gchar *set_base;
        gchar *anchor = NULL;
        gchar *conv_link;
        GtkHTMLStream *handle;
        
        full_link = chmsee_html_split_uri(str_uri, &anchor);
        
        /* same page */
        if (chmsee->current_link) {
                if (g_strcasecmp(chmsee->current_link, full_link) == 0 ) {
                        if (anchor) {
                                gtk_html_jump_to_anchor(GTK_HTML (chmsee->chm_html), anchor);
                        }
                        
                        return;
                }
        }
        
        conv_link = g_strdup(chmsee_conv_uri(full_link));

        if (!g_file_test(g_strdown(conv_link), G_FILE_TEST_EXISTS)) {
                open_uri_errord();
                g_free (anchor);
                g_free (full_link);
                g_free (conv_link);
                return;
        }

        chmsee_add_list(chmsee, (gchar *)str_uri, i);

        handle = gtk_html_begin_content(GTK_HTML (chmsee->chm_html), "text/html; charset=utf-8");
        set_base = g_strdup_printf("%s/", g_path_get_dirname(conv_link));
        gtk_html_set_base(GTK_HTML (chmsee->chm_html), set_base);
        chmsee->current_link = g_strdup(str_uri);
        chmsee_html_url_requested(GTK_HTML (chmsee->chm_html), g_strdown(conv_link), handle, chmsee);

        if (anchor) {
                gtk_html_jump_to_anchor(GTK_HTML (chmsee->chm_html), anchor);
        }

        g_free(conv_link);
        g_free(set_base);
}

void
chmsee_html_reload(ChmSee *chmsee, const gchar *str_uri)
{
        gchar *full_link;
        gchar *anchor = NULL;;
        gchar *conv_link;
        GtkHTMLStream *handle;

        if (str_uri) {
                full_link = chmsee_html_split_uri(str_uri, &anchor);
                handle = gtk_html_begin_content(GTK_HTML(chmsee->chm_html), "text/html; charset=utf-8");
                conv_link = g_strdup(chmsee_conv_uri(full_link));
                chmsee_html_url_requested(GTK_HTML (chmsee->chm_html), g_strdown(conv_link), handle, chmsee);
                
                if (anchor) {
                        gtk_html_jump_to_anchor(GTK_HTML (chmsee->chm_html), anchor);
                }
                
                g_free(conv_link);
        }
}

void
chmsee_html_load_empty(ChmSee *chmsee)
{
        gtk_html_load_empty( GTK_HTML( chmsee->chm_html ) );
}

GtkWidget* 
chmsee_html_view_new(ChmSee *chmsee)
{
        GtkWidget *html;

        html = gtk_html_new();
        gtk_html_set_allow_frameset (GTK_HTML(html), TRUE);

        g_signal_connect(GTK_OBJECT (html), 
                         "load_done", 
                         G_CALLBACK(chmsee_html_load_done), 
                         NULL);

        g_signal_connect(GTK_OBJECT (html), 
                         "size_changed", 
                         G_CALLBACK(chmsee_html_refresh), 
                         NULL);

        g_signal_connect(GTK_OBJECT (html), 
                         "link_clicked", 
                         G_CALLBACK (chmsee_html_link_clicked), 
                         chmsee);

        g_signal_connect(GTK_OBJECT (html), 
                         "button_press_event",
                         G_CALLBACK (chmsee_html_button_pressed), 
                         chmsee);
        
        g_signal_connect(GTK_OBJECT (html), 
                         "url_requested", 
                         G_CALLBACK (chmsee_html_url_requested), 
                         chmsee);

        return (html);
}
