/*
 *  Copyright (c) 2006           Ji YongGang <jungle@soforge.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/***************************************************************************
 *   Copyright (C) 2003 by zhong                                           *
 *   zhongz@163.com                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <string.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <gtkhtml/gtkhtml-search.h>

#include "chmsee.h"
#include "search.h"

typedef struct _ChmSee_Search ChmSee_Search;

struct _ChmSee_Search {
        GtkWidget       *search_chm_html;
        GtkWidget       *search_entry;
        GtkWidget       *search_checkbutton;
        GtkWidget       *button2;
        GtkWidget       *button3;
};

static void chmsee_html_search(GtkWidget *, ChmSee_Search *);
static void chmsee_html_search_next(GtkWidget *, ChmSee_Search *);
static void on_search_window_destroy(GtkWidget *, ChmSee_Search *);
static void search_entry_changed(GtkEntry *, ChmSee_Search *);

static void 
chmsee_html_search(GtkWidget *widget, ChmSee_Search *sh_window)
{
        gchar *text;
        gchar *text_utf;
        gboolean toggle_button;

        text = g_strdup_printf("%s", gtk_entry_get_text(GTK_ENTRY(sh_window->search_entry)));
        text_utf = g_locale_from_utf8(text, -1, NULL, NULL, NULL);
        LOG_DEBUG ("get text->[%s]", text_utf);
        toggle_button = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (sh_window->search_checkbutton));
        gtk_html_engine_search (GTK_HTML( sh_window->search_chm_html ), 
                                (const gchar *)text_utf, toggle_button, TRUE, FALSE);
        g_free(text);
}

static void 
chmsee_html_search_next(GtkWidget *widget, ChmSee_Search *sh_window)
{
        gtk_html_engine_search_next (GTK_HTML( sh_window->search_chm_html ));
}

static void
on_search_window_destroy(GtkWidget *widget, ChmSee_Search * sh_window)
{
        gtk_widget_destroy(widget);
        g_free(sh_window);
}

static void 
search_entry_changed (GtkEntry *entry, ChmSee_Search *sh_window)
{
        const gchar *text;
        gint text_length;

        text = gtk_entry_get_text (GTK_ENTRY (entry));
        text_length = strlen(text);
        
        if (text_length >= 2) {
                gtk_widget_set_sensitive(sh_window->button2, TRUE);
                gtk_widget_set_sensitive(sh_window->button3, TRUE);
        } else {
                gtk_widget_set_sensitive(sh_window->button2, FALSE);
                gtk_widget_set_sensitive(sh_window->button3, FALSE);
        }
}

void 
create_search_window(GtkWidget *widget, ChmSee *chmsee)
{
        GladeXML *glade;
        ChmSee_Search *sh_window;
        sh_window = g_new0(ChmSee_Search, 1);

        GtkWidget *search_window;
        GtkWidget *button1;

        /* create setup window */
        glade = glade_xml_new(CHMSEE_DATA_DIR"/"GLADE_FILE, "search_window", NULL);
        search_window = glade_xml_get_widget(glade, "search_window");
        g_signal_connect(search_window, 
                         "destroy",
                         G_CALLBACK (on_search_window_destroy),
                         sh_window);

        sh_window->search_chm_html = chmsee->chm_html;
        
        sh_window->search_entry = glade_xml_get_widget(glade, "search_entry");

        g_signal_connect(sh_window->search_entry, 
                          "changed", 
                          G_CALLBACK (search_entry_changed),
                          sh_window);

        sh_window->search_checkbutton = glade_xml_get_widget(glade, "search_checkbutton");

        button1 = glade_xml_get_widget(glade, "button1");
        g_signal_connect_swapped((gpointer) button1, 
                                 "clicked",
                                 G_CALLBACK (gtk_widget_destroy),
                                 GTK_OBJECT (search_window));

        sh_window->button2 = glade_xml_get_widget(glade, "button2");
        g_signal_connect((gpointer) sh_window->button2, 
                         "clicked",
                         G_CALLBACK (chmsee_html_search),
                         sh_window);

        sh_window->button3 = glade_xml_get_widget(glade, "button3");
        g_signal_connect((gpointer) sh_window->button3, 
                         "clicked",
                         G_CALLBACK (chmsee_html_search_next),
                         sh_window);

        gtk_widget_show(search_window);
}
