/*
 *  Copyright (c) 2006           Ji YongGang <jungle@soforge.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>

#include "chmsee.h"
#include "utils.h"

gchar *
convert_filename_to_utf8(const gchar *filename, const gchar *codeset)
{
        gchar * filename_utf8;
        gchar * filename_lowcase;

        if (g_utf8_validate(filename, -1, NULL)) {
                LOG_INFO("filename is utf8");
                filename_utf8 = g_strdup(filename);
        } else {
                LOG_INFO("filename is not utf8");
                filename_utf8 = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);

                if (!filename_utf8) {
                        filename_utf8 = g_convert(filename, -1, "UTF-8", 
                                                  codeset, 
                                                  NULL, NULL, NULL);
                }
        }

        filename_lowcase = g_utf8_strdown(filename_utf8, -1);

        g_free(filename_utf8);

        return filename_lowcase;
}

gint
ncase_compare_utf8_string(const gchar *str1, const gchar *str2)
{
        gint result;
        gchar *ncase_str1, *ncase_str2;
        gchar *normalized_str1, *normalized_str2;

        ncase_str1 = g_utf8_casefold(str1, -1);
        ncase_str2 = g_utf8_casefold(str2, -1);

        normalized_str1 = g_utf8_normalize(ncase_str1, -1, G_NORMALIZE_DEFAULT);
        normalized_str2 = g_utf8_normalize(ncase_str2, -1, G_NORMALIZE_DEFAULT);

        result = g_utf8_collate(normalized_str1, normalized_str2);

        g_free(ncase_str1);
        g_free(ncase_str2);
        g_free(normalized_str1);
        g_free(normalized_str2);

        return result;
}
